package
{
	import defines.Assets;

	import map.MapInfo;
	import map.HexagonMap;
	import map.HexagonTile;
	
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.utils.Key;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.utils.Input;
	import net.flashpunk.World;
	import net.flashpunk.Sfx;
	
	import flash.geom.Point;
	import flash.system.System;
	
	public class GameEditor extends GameLevels
	{
		private var mTinyKid:TinyKid = new TinyKid();
		//mSelectedKid from GameWorld is used to place kids in world.
		private var mEditLevel:int = 0;
		public static var Instance:GameEditor;
		
		public function GameEditor():void
		{
			Instance = this;
			LoadLevel(mEditLevel);
			ToggleMute();
			
			add(mTinyKid);
			mTinyKid.x = 32;	mTinyKid.y = 500;
		}
		
		override public function IsEditing():Boolean { return true; }
		
		public function ReadyToSave():Boolean { return Input.check(Key.SHIFT) && Input.check(Key.S); }
		public function MakeSaveString():String
		{
			var saveString:String = mWorldMap.editorSaveString();
			saveString += "mWorldKids = new Array("
			for (var i:int = 0; i < mWorldKids.length; ++i)
			{
				if (i != 0)	saveString += ", ";
				saveString += "new TinyKid(" + mWorldKids[i].x + ", " + mWorldKids[i].y + ")";
			}
			saveString += ");\n";
			
			saveString += "mActionsPerRound = " + mActionsPerRound + ";		mMinSafeKidsToUnlock = " + mMinSafeKidsToUnlock + ";\n";
			return saveString;
		}
		
		override public function update():void
		{
			super.update();
			
			if (true == Input.check(Key.ESCAPE))
				FP.world = new LevelSelect();
			
			var mousePt:Point = new Point(FP.world.mouseX, FP.world.mouseY);
			var selectedTile:HexagonTile = mWorldMap.CollisionWithMap(mousePt);

			if (true == Input.mousePressed)
			{	//Click-and-toggle Editor Mode.
				if (null != selectedTile)
				{
					if (Input.check(Key.SHIFT))
						mWorldMap.GetTileInfo(selectedTile.idx).type = (selectedTile.type - 1) % HexagonMap.NUM_TILE_TYPES;
					else
						mWorldMap.GetTileInfo(selectedTile.idx).type = (selectedTile.type + 1) % HexagonMap.NUM_TILE_TYPES;
				}
				else if (mTinyKid.distanceToPoint(mousePt.x, mousePt.y) < 12)
				{
					mSelectedKid = mTinyKid;
				}
			}
			else if (true == Input.mouseReleased && null != mSelectedKid)
			{	//If the kid is over a tile, duplicate and put into Kid list, then put the selector object back.
				if (null != selectedTile && CanMoveTo(-1, selectedTile))
				{
					var kid:TinyKid = new TinyKid();
					add(kid);
					mWorldKids.push(kid);
					MoveKidTo(mWorldKids.length - 1, selectedTile);
				}
				
				mSelectedKid.x = 32;	mSelectedKid.y = 500;
				mSelectedKid = null;
			}

			if (null != mSelectedKid)
			{
				mSelectedKid.x = mouseX;
				mSelectedKid.y = mouseY;
			}
			
			var mouseDelta:int = Input.mouseWheelDelta;
			if (mouseDelta > 1)		mouseDelta = 1;
			if (mouseDelta < -1)	mouseDelta = -1;
			
			if (0 != mouseDelta && null != selectedTile)
			{
				if (mouseDelta < 0)
				{
					if (mWorldMap.GetTileInfo(selectedTile.idx).life > -1)
						mWorldMap.GetTileInfo(selectedTile.idx).life -= 1;
				}
				else
					mWorldMap.GetTileInfo(selectedTile.idx).life += 1;
			}
			else if (0 != mouseDelta)
			{
				if (mouseX > 10 && mouseX < 150 && mouseY > 570 && mouseY < 590)
				{
					mActionsPerRound += mouseDelta;
					if (mActionsPerRound < 1)
						mActionsPerRound = 1;
				}
				if (mouseX > 10 && mouseX < 150 && mouseY > 550 && mouseY < 570)
				{
					mMinSafeKidsToUnlock += mouseDelta;
					if (mMinSafeKidsToUnlock < 1)
						mMinSafeKidsToUnlock = 1;
				}
			}
			
			if (true == Input.check(Key.CONTROL))
			{
				var idx:int = 0;
				if (true == Input.pressed(Key.N))
				{
					ClearLevel();
					mWorldMap.editorLoadDefault();
				}
				if (true == Input.pressed(Key.S) || true == Input.pressed(Key.C))
				{
					var saveString:String = MakeSaveString();
					System.setClipboard(saveString);
					//var outFile:FileReference = new FileReference();
					//outFile.save(mapDataAsString);
				}
				if (true == Input.pressed(Key.K))
				{
					ClearAllKids();
				}
				if (true == Input.pressed(Key.UP))
				{
					mEditLevel += 1;
					LoadLevel(mEditLevel);
				}
				if (true == Input.pressed(Key.DOWN))
				{
					mEditLevel -= 1;
					LoadLevel(mEditLevel);
				}
			}
		}
			
		override public function render():void
		{
			super.render();
			mWorldMap.renderEditor(FP.buffer, new Point(0, 0), camera);
			
			var txt:Text = new Text("Editing Level: " + mEditLevel, 10, 510);
			txt.render(FP.buffer, new Point(0, 0), camera);
			
			txt.text = mWorldKids.length.toString() + " kids Min Safe: " + mMinSafeKidsToUnlock;	txt.y = 550;
			txt.render(FP.buffer, new Point(0, 0), camera);
			
			txt.text = "Actions per Round: " + mActionsPerRound;	txt.y = 570;
			txt.render(FP.buffer, new Point(0, 0), camera);
		}
	}
}