/*~******************************************************************************************
*	~Project:	"BusTown"																	*
*	~Author:	Tim "BlackBird" Beaudet														*
*	~Modified:	1/13/2012																	*
******************************************************************************************~*/

package
{
	import defines.Assets;
	import defines.TileTypes;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import net.flashpunk.Entity;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.tweens.misc.NumTween;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.utils.Key;
	import net.flashpunk.utils.Input;
	import net.flashpunk.World;
	import net.flashpunk.Sfx;
	import ui.MuteSoundsBtn;
	
	import ui.GameTutorial;
	import ui.PopupBox;
	import ui.RestartBtn;
	import ui.DoneActionsBtn;
	import ui.BasicBtn;
	import ui.Display;

	import gameobjs.FallingTile;
	import gameobjs.RemoveAction;
	
	import map.MapInfo;
	import map.HexagonMap;
	import map.HexagonTile;
	
	public class GameWorld extends World
	{
		private static const DAY_NIGHT_TIME:Number = 1.0;
		private static const MIN_DISTANCE_TO_KID:Number = 11;
		private static var WorldInstance:GameWorld = null;
		public static var UnlockedLevels:int = 1;
		private static var TotalKidsSaved:int = 0;
		
		private static var dbgPlayerMove:Boolean = true;
		
		protected var mCurrLevel:int = 0;
		protected var mWorldMap:HexagonMap = new HexagonMap(Assets.HEXAGON_TILESET_SD, 32);
		protected var mMapOverlay:HexagonMap = new HexagonMap(Assets.HEXAGON_TILESET_SD, 32);
		protected var mWorldKids:Array = new Array();
		
		protected var mSelectedKid:TinyKid = null;
		protected var mSelectedFrom:HexagonTile = null;
		protected var mActionsLeft:int = 3;
		protected var mDisplayedActionsLeft:int = 3;
		protected var mActionsPerRound:int = 3;
		protected var mActionsUnused:int = 0;
		protected var mActionsUsed:int = 0;
		protected var mRoundsPlayed:int = 0;
		protected var mKidsKeptSafe:int = 0;
		protected var mMinSafeKidsToUnlock:int = 1;
		protected var mNumKidsAtStart:int = 0;

		protected var mDayNightOverlay:Image = new Image(Assets.GAME_INTERFACE, new Rectangle(400, 440, 85, 65));
		protected var mNightToDayTimer:Number = -1.0;
		protected var mDayToNightTimer:Number = -1.0;
		
		//PostRound Update stuff.
		public static var WaitToFinish:int = 0;
		private var mNeedToBegin:Boolean = false;

		protected var mTileCrackTimer:Number = -1.0;
		protected var mCrackingTiles:Array = new Array();		//If this is not empty, post-round animations should be playing out.
		
		public static var mGameMusic:Sfx = new Sfx(Assets.MUSIC_BACKGROUND);
		
		public static function GetInstance():GameWorld { return WorldInstance; }
		
		public function GameWorld():void
		{
			WorldInstance = this;
		
			add(new CameraShake());
			addGraphic(mWorldMap);
			addGraphic(mMapOverlay);		mMapOverlay.visible = false;
			LoadLevel(1);
			
			if (false == IsEditing())
			{
				add(new Display(this));
				add(new RestartBtn(750, 550));
				add(new DoneActionsBtn(50, 550));
				add(new MuteSoundsBtn());
				add(new GameTutorial(this));

				if (false == mGameMusic.playing)
					mGameMusic.loop();
			}
			
			add(new PopupBox);
			
			mDayNightOverlay.scale = 10;
			mDayNightOverlay.centerOrigin();
			mDayNightOverlay.x = FP.screen.width / 2;
			mDayNightOverlay.y = FP.screen.height / 2;
		}
		
		public function GetTotalActionsUsed():int { return mActionsUsed; }
		public function GetTotalActionsUnused():int { return mActionsUnused; }
		public function GetTotalNumberKidsSaved():int { return TotalKidsSaved; }
		public function GetNumKidsToSave():int { return mMinSafeKidsToUnlock; }
		public function GetNumKidsSaved():int { return mKidsKeptSafe; }
		public function GetNumKidsAtStart():int { return mNumKidsAtStart; }
		public function GetDisplayedActionsLeft():int { return mDisplayedActionsLeft; }
		public function SetDisplayedActionsLeft(left:int):void { mDisplayedActionsLeft = left; }
		public function GetCurrentLevel():int { return mCurrLevel; }
		public function GetActionsPerRound():int { return mActionsPerRound; }
		public function GetActionsLeft():int { return mActionsLeft; }
		public function GetRoundsPlayed():int { return mRoundsPlayed; }
		
		private function OnLevelComplete():void
		{
			if (Assets.IS_DEV_MODE && true == IsEditing()) { return; }
			
			//For one reason or another we are here.  Detect win/lose, unlock levels etc and then show results.
			var winner:Boolean = false;
			if (mKidsKeptSafe >= mMinSafeKidsToUnlock)
			{	//Unlock the next level, but don't lock the above ones if the player is replaying!
				if (UnlockedLevels <= mCurrLevel)
					UnlockedLevels = mCurrLevel + 1;
				winner = true;
			}
			
			var res:GameResults = new GameResults(winner);
			FP.world = res;
		}
		
		private function OnRoundBegin():void
		{
			mActionsLeft = mActionsPerRound;
			mDisplayedActionsLeft = mActionsLeft;
			
			if (0 == mWorldKids.length)
			{	//Game has no more kids left.  All have been saved or killed.
				OnLevelComplete();
			}
		}
		
		private function OnRoundFinish():void
		{
			mRoundsPlayed++;
			for (var r:int = 0; r < mWorldMap.GetNumRows(); ++r)
			{
				for (var c:int = 0; c < mWorldMap.GetNumCols(); ++c)
				{
					var info:MapInfo = mWorldMap.GetTileInfo(mWorldMap.mapIndexOf(c, r));
					if (info.life > 4)
						info.life--;
					else if (info.life > 0)
						mCrackingTiles.push(mWorldMap.GetTileAt(c, r));
				}
			}
	
			mDayToNightTimer = DAY_NIGHT_TIME;
			mNeedToBegin = true;
		}
		
		public function SkipRemainingActions():void
		{
			mActionsUnused += mActionsLeft;
			OnPerformAction(null, mActionsLeft);
		}
		
		//Returns true if the round needs to finish now.  No actions remain.
		private function OnPerformAction(fromPos:Point, numActs:int = 1):Boolean
		{
			//if (null != fromPos)
			//{
				//const R:Number = 80;
				//const hR:Number = R * 0.5;
				//
				//for (var i:int = 0; i < numActs; ++i)	//These objects subtract from display.
					//add(new RemoveAction(fromPos.x + (Math.random() * R) - hR, fromPos.y + (Math.random() * R) - hR));
			//}
			//else
			{ mDisplayedActionsLeft -= numActs; }
			
			mActionsLeft -= numActs;
			mActionsUsed += numActs;
			if (mActionsLeft <= 0)
			{
				OnRoundFinish();
				return true;
			}
			
			if (false == CanPlayerPerformAction())
			{
				if (0 != mWorldKids.length)
				{
					PopupBox.ShowMsg("No actions possible.\n\n\tEnding the current day\n\tautomatically.");
					OnRoundFinish();
				}
				else
				{
					OnLevelComplete();
				}
				return true;
			}
			
			return false;
		}
		
		private function CanPlayerPerformAction():Boolean
		{	//This is the beast...
			for (var kidIdx:int = 0; kidIdx < mWorldKids.length; ++kidIdx)
			{
				var fromTile:HexagonTile = mWorldMap.CollisionWithMap(new Point(mWorldKids[kidIdx].x, mWorldKids[kidIdx].y));	//Should never be null anyway.
				
				for (var row:int = 0; row < mWorldMap.GetNumRows(); ++row)
				{
					for (var col:int = 0; col < mWorldMap.GetNumCols(); ++col)
					{
						var toTile:HexagonTile = mWorldMap.GetTileAt(col, row);
						var toInfo:MapInfo = mWorldMap.GetTileInfo(toTile.idx);
						var cost:int = GetCostFor(toInfo.type, toInfo.subType);
						if (toTile.idx != fromTile.idx && true == CanMoveFromTo(kidIdx, fromTile, toTile) && true == CanMoveTo(kidIdx, toTile))
							return true;
					}
				}
			}
			return false;
		}
		
		public function CollisionWithKids(atX:Number, atY:Number):TinyKid
		{
			for (var idx:int = 0; idx < mWorldKids.length; ++idx)
			{
				if (mWorldKids[idx].distanceToPoint(atX, atY) < MIN_DISTANCE_TO_KID)
					return mWorldKids[idx];
			}
			return null;
		}
		
		public function IsEditing():Boolean { return false; }
		public function FindKidIndex(kid:TinyKid):int { for (var i:int = 0; i < mWorldKids.length; ++i) if (mWorldKids[i] == kid) return i; return -1; }

		public override function update():void
		{
			if (false == PopupBox.IsShowing() && Input.pressed(Key.ESCAPE))
				FP.world = new LevelSelect();

			super.update();
			if (Input.pressed(Key.M))
				ToggleMute();
					
			if (true == IsEditing() || true == PopupBox.IsShowing() || true == updatePostRound())
			{
				dbgPlayerMove = false;
				return;
			}
			
			if (false == dbgPlayerMove)
				trace("Playet Movement Unlocked.");
			dbgPlayerMove = true;
			
			var mousePt:Point = new Point(mouseX, mouseY);
			if (true == Input.mousePressed)
			{
				var kid:TinyKid = CollisionWithKids(mouseX, mouseY);
				if (null != kid)
				{
					mSelectedKid = kid;
					mSelectedKid.OnGrab();
					mSelectedFrom = mWorldMap.CollisionWithMap(mousePt);
					mMapOverlay.visible = true;
					updateMapOverlay(FindKidIndex(mSelectedKid), mSelectedFrom);
				}
			}
			if (true == Input.mouseReleased && null != mSelectedKid)
			{	//Do stuff like, make sure the player can move here (mouseX, mouseY).
				var kidIdx:int = FindKidIndex(mSelectedKid);
				var toTile:HexagonTile = mWorldMap.CollisionWithMap(mousePt);
				var toInfo:MapInfo = mWorldMap.GetTileInfo(toTile.idx);
				var cost:int = GetCostFor(toInfo.type, toInfo.subType);
				var sameTile:Boolean = (toTile.idx == mSelectedFrom.idx) ? true : false;
				var moved:Boolean = true;
				if (false == sameTile && true == CanMoveFromTo(kidIdx, mSelectedFrom, toTile) && true == CanMoveTo(kidIdx, toTile))
				{
					RemoveKidFromTile(kidIdx, mSelectedFrom);
					MoveKidTo(kidIdx, toTile);
					
					if (toInfo.type == 4)	//This would be the 'HOUSE' tile.
					{
						//remove(mWorldKids[kidIdx]);		//Let the kid remove themself after an animation to celebrate.
						mWorldKids[kidIdx].OnReachSafety();
						bringToFront(mWorldKids[kidIdx]);
						mWorldKids.splice(kidIdx, 1);
						mKidsKeptSafe++;
						TotalKidsSaved++;
						WaitToFinish++;
					}
					
					OnPerformAction(toTile.pos, cost);
				}
				else
				{
					moved = false;
					MoveKidTo(kidIdx, mSelectedFrom);
				}
				
				//Deselect the kid.
				mSelectedKid.OnUngrab(moved, (moved) ? (cost == 1) : sameTile);
				mSelectedKid = null;
				mSelectedFrom = null;
				mMapOverlay.visible = false;
			}
			if (null != mSelectedKid)
			{
				mSelectedKid.x = mousePt.x;
				mSelectedKid.y = mousePt.y;
			}
		}
		
		private function updatePostRound():Boolean
		{
			const TIME_PER_TILE:Number = 0.05;
			if (mDayToNightTimer >= 0.0)
			{
				mDayToNightTimer -= FP.elapsed;
				mNightToDayTimer = DAY_NIGHT_TIME;
				return true;	//We are busy here.
			}
			
			if (0 == mCrackingTiles.length && 0 == WaitToFinish && mNightToDayTimer < 0.0)
			{
				if (true == mNeedToBegin)
				{	OnRoundBegin();		mNeedToBegin = false;  }
				return false;
			}
			
			var prevTime:Number = mTileCrackTimer;
			mTileCrackTimer -= FP.elapsed;
			
//			This wasn't stable enough?
//			var crackTime:Number = TIME_PER_TILE * 0.5;
//			if (prevTime >= crackTime && mTileCrackTimer < crackTime)
//				mWorldMap.GetTileInfo(mCrackingTiles[mCrackingTiles.length - 1].idx).life--;
			
			if (mTileCrackTimer < 0.0)
			{	//TODO: The timer may not be active for the first element?
				if (0 != mCrackingTiles.length)
				{
					var crackingTile:HexagonTile = mCrackingTiles[mCrackingTiles.length - 1];
					mWorldMap.GetTileInfo(crackingTile.idx).life--;
					if (mWorldMap.GetTileInfo(crackingTile.idx).life == 0)
					{
						Assets.PlaySoundArr(Assets.arrSfxCrackingBig);
						mWorldMap.GetTileInfo(crackingTile.idx).type = TileTypes.Empty;
						var fallingTile:FallingTile = new FallingTile(crackingTile, mWorldMap.GetTileInfo(crackingTile.idx), mWorldMap);
						add(fallingTile);
						
						KillKidsOnTile(crackingTile);
						WaitToFinish++;

						CameraShake.SetShakeTimer(0.5);		CameraShake.SetShakeAmount(6);
					}
					else
					{
						if (true == CameraShake.IsShaking() && CameraShake.GetShakeAmount() > 0.25)
						{	CameraShake.SetShakeTimer(CameraShake.GetShakeTimer() + 0.1);	}
						else
						{	CameraShake.SetShakeTimer(0.25);	CameraShake.SetShakeAmount(2.5);	}
						Assets.PlaySoundArr(Assets.arrSfxCrackingSmall);
					}
						
					mCrackingTiles.pop();
					if (mCrackingTiles.length)
					{	//Setup to play with the next cracking tile.
						mTileCrackTimer += TIME_PER_TILE;		//Time to spend per tile cracking or falling..
					}
					
					mNightToDayTimer = DAY_NIGHT_TIME;
				}
				
				//if (mCrackingTiles.length)
				//{	//Setup to play with the next cracking tile.
					//mTileCrackTimer += TIME_PER_TILE;		//Time to spend per tile cracking or falling..
				//}
			}
			
			if (mNightToDayTimer >= 0.0)
			{
				mNightToDayTimer -= FP.elapsed;
				return true;
			}
			
			return true;
		}
		
		public function updateMapOverlay(kidIdx:int, fromTile:HexagonTile):void
		{
			mMapOverlay.SetAllTilesTo(0);
			for (var r:int = 0; r < mWorldMap.GetNumRows(); ++r)
			{
				for (var c:int = 0; c < mWorldMap.GetNumCols(); ++c)
				{
					var toTile:HexagonTile = mWorldMap.GetTileAt(c, r);
					var toInfo:MapInfo = mWorldMap.GetTileInfo(toTile.idx);
					if (CanMoveFromTo(kidIdx, fromTile, toTile))
					{
						var cost:int = GetCostFor(toInfo.type, toInfo.subType);
						if (CanMoveTo(kidIdx, toTile))
						{
							if (cost > 1)
								mMapOverlay.SetTileType(mMapOverlay.mapIndexOf(c, r), 24);
							else
								mMapOverlay.SetTileType(mMapOverlay.mapIndexOf(c, r), 8);
						}
						else
						{
							if (toInfo.type != 0 && toInfo.life != 0)	//Ignore the empty and dead tiles.
								mMapOverlay.SetTileType(mMapOverlay.mapIndexOf(c, r), 16);
						}
					}
				}
			}
		}
		
		public override function render():void
		{
			super.render();
			
			if (true == Assets.IS_DEV_MODE)
			{
				var txt:Text = new Text("FPS: " + FP.frameRate.toPrecision(3), 10, 10);
				txt.render(FP.buffer, new Point(0, 0), camera);
			}
			
			if (mDayToNightTimer >= 0.0 || mNightToDayTimer >= 0.0)
			{
				var p:Number = 0.0;
				if (mDayToNightTimer >= 0.0)
					p = 1.0 - (mDayToNightTimer / DAY_NIGHT_TIME);
				else if (mNightToDayTimer >= 0.0)
					p = (mNightToDayTimer / DAY_NIGHT_TIME);
									
				if (p > 0.0)
				{
					if (p > 1.0)	p = 1.0;
					mDayNightOverlay.alpha = p * 0.5;
					
					//var al:Number = 0.5 * p;
					//var alpha:uint = (int(al * 256.0));
					//var alpha:uint = 0x80;
					//var color:uint = 0x00202020 & (alpha<<24);
					//FP.buffer.fillRect(new Rectangle(0, 0, FP.screen.width, FP.screen.height), color);
					
					mDayNightOverlay.render(FP.buffer, new Point(0, 0), camera);
				}
			}
		}
		
		private function AnyTilesAlive():Boolean
		{
			for (var r:int = 0; r < mWorldMap.GetNumRows(); ++r)
			{
				for (var c:int = 0; c < mWorldMap.GetNumCols(); ++c)
				{
					if (mWorldMap.GetTileInfo(mWorldMap.mapIndexOf(c, r)).life > 0)
						return true;
				}
			}
			return false;
		}
		
		private function AnyKidsAlive():Boolean
		{
			return (0 == mWorldKids.length) ? false : true;
		}
		
		private function KillKidsOnTile(onTile:HexagonTile):void
		{
			var kidsOnTile:Array = GetKidsOnTile(onTile);
			for (var i:int = 0; i < kidsOnTile.length; ++i)
			{
				var delKid:int = mWorldKids.indexOf(kidsOnTile[i]);
				
				//Instead of:
				////remove(mWorldKids[delKid]);
				//mWorldKids[delKid].OnDeath();
				
				//We remove the delKid and make a new one to kill.
				remove(mWorldKids[delKid]);
				var deadKid:TinyKid = new TinyKid(mWorldKids[delKid].x, mWorldKids[delKid].y);
				deadKid.OnDeath();
				add(deadKid);
				

				WaitToFinish++;
				mWorldKids.splice(delKid, 1);
				trace("Killed kid.  Num rem: " + mWorldKids.length);
			}
		}
		
		public static function ToggleMute():void
		{
			FP.volume = (false == IsMuted()) ? 0.0 : 1.0;
		}
		
		public static function IsMuted():Boolean
		{
			return (FP.volume >= 0.1) ? false : true;
		}
		
		public static function GetCostFor(tileType:int, subType:int):int
		{
			var cost:int = 1;
			switch (tileType)
			{
			case 0: cost = -1; break;	//Empty
			case 2: cost = 2; break;	//Grass
			case 3: cost = -1; break;	//Mountains
			default: cost = 1; break;
			};
			
			//switch (subType)
			//{
				//case X: cost += 2; break;
			//};
			return cost;
		}
		
		public function CanMoveTo(idx:int, tile:HexagonTile, costCheck:Boolean = true):Boolean
		{
			if (tile == null)
				return false;		//Can't move here, tile does not exist.
			
			var info:MapInfo = mWorldMap.GetTileInfo(tile.idx);
			if (GetCostFor(info.type, info.subType) == -1)
				return false;		//Can't move onto this tile type.
				
			if (true == costCheck && GetCostFor(info.type, info.subType) > mActionsLeft)
				return false;		//Can't move, not enough actions.
				
			if (info.life == 0)
				return false;		//The tile is gone, no can move.
				
			var headCount:int = 0;
			for (var i:int = 0; i < mWorldKids.length; ++i)
			{
				if (idx != i && true == mWorldMap.CollidesWithTile(new Point(mWorldKids[i].x, mWorldKids[i].y), tile.col, tile.row))
					headCount++;
			}
			
			if (headCount >= 3)
				return false;		//Can't move here, tile is full.

			return true;
		}
		
		public function CanMoveFromTo(idx:int, fromTile:HexagonTile, toTile:HexagonTile):Boolean
		{
//			if (false == CanMoveTo(idx, pt))
//				return false;
//			if (false == CanMoveFrom(idx, fromTile.pos))
//				return false;
			
//			var toTile:HexagonTile = mWorldMap.CollisionWithMap(pt);
			
			if (null == toTile || null == fromTile || toTile == fromTile)
				return false;
				
			var diffCol:int = toTile.col - fromTile.col;
			var diffRow:int = toTile.row - fromTile.row;
			//trace("Attempting to move from " + fromTile.col + ", " + fromTile.row + " to " + toTile.col + ", " + toTile.row);
			//trace("    diff: " + diffCol + ", " + diffRow);
			
			if (Math.abs(diffCol) > 1 || Math.abs(diffRow) > 1)
				return false;
			if (fromTile.row % 2 == 0 && diffCol > 0 && diffRow != 0)
				return false;
			if (fromTile.row % 2 == 1 && diffCol < 0 && diffRow != 0)
				return false;
		
			return true;
		}
		
		private function RepositionKidsOnTile(kidsOnTile:Array, tile:HexagonTile):void
		{	//Reposition all the kids on the tile to be neat.
			switch (kidsOnTile.length)
			{
			case 1:
				kidsOnTile[0].x = tile.pos.x;
				kidsOnTile[0].y = tile.pos.y;
				break;
			case 2:
				kidsOnTile[0].x = tile.pos.x + 13;
				kidsOnTile[0].y = tile.pos.y;
				kidsOnTile[1].x = tile.pos.x - 13;
				kidsOnTile[1].y = tile.pos.y;
				break;
			case 3:
				kidsOnTile[0].x = tile.pos.x + 13;
				kidsOnTile[0].y = tile.pos.y + 5;
				kidsOnTile[1].x = tile.pos.x - 13;
				kidsOnTile[1].y = tile.pos.y + 5;
				kidsOnTile[2].x = tile.pos.x;
				kidsOnTile[2].y = tile.pos.y - 12;
				break;
			case 4:
				kidsOnTile[0].x = tile.pos.x - 13;
				kidsOnTile[0].y = tile.pos.y + 13;
				kidsOnTile[1].x = tile.pos.x + 13;
				kidsOnTile[1].y = tile.pos.y - 13;
				kidsOnTile[2].x = tile.pos.x - 13;
				kidsOnTile[2].y = tile.pos.y - 13;
				kidsOnTile[3].x = tile.pos.x + 13;
				kidsOnTile[3].y = tile.pos.y + 13;
				break;
			};
			
		}
		
		public function GetKidsOnTile(onTile:HexagonTile, skipIdx:int = -1):Array
		{
			var kidsOnTile:Array = new Array();
			for (var i:int = 0; i < mWorldKids.length; ++i)
			{
				if ((skipIdx == -1 || i != skipIdx) && true == mWorldMap.CollidesWithTile(new Point(mWorldKids[i].x, mWorldKids[i].y), onTile.col, onTile.row))
					kidsOnTile.push(mWorldKids[i]);
			}
			return kidsOnTile;
		}

		public function RemoveKidFromTile(idx:int, fromTile:HexagonTile):Boolean
		{
			var kidsOnTile:Array = GetKidsOnTile(fromTile, idx);
			RepositionKidsOnTile(kidsOnTile, fromTile);
			return true;
		}
		
		public function MoveKidTo(idx:int, toTile:HexagonTile):Boolean
		{
			//var toTile:HexagonTile = mWorldMap.CollisionWithMap(pt);

			if (false == CanMoveTo(idx, toTile, false))
				return false;		//Tile-type, full, off-map; for some reason can't go there.
				
			var kidsOnTile:Array = GetKidsOnTile(toTile, idx);
			kidsOnTile.push(mWorldKids[idx]);
			RepositionKidsOnTile(kidsOnTile, toTile);
			
			return true;
		}
		
		public function ClearAllKids():void
		{
			for (var i:int = 0; i < mWorldKids.length; ++i)
				remove(mWorldKids[i]);
			mWorldKids = new Array();
		}
		public function ClearLevel():void
		{
			mKidsKeptSafe = 0;
			mRoundsPlayed = 0;
			mActionsUnused = 0;
			mActionsUsed = 0;
			WaitToFinish = 0;
			ClearAllKids();
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		public function ReloadLevel():void
		{
			LoadLevel(mCurrLevel);
		}
		
		public function LoadLevel(lvlIdx:int = 1):void
		{
			mCurrLevel = lvlIdx;

			//Finalize the loading prcoess.
			mNumKidsAtStart = mWorldKids.length;
			for (var i:int = 0; i < mWorldKids.length; ++i)
				add(mWorldKids[i]);
				
			//First round is beginning.
			OnRoundBegin();
			
			CameraShake.SetHoldPosition(camera.x, camera.y);
		}
	}
}