/*~******************************************************************************************
*	~Project:	"BusTown"																	*
*	~Author:	Tim "BlackBird" Beaudet														*
*	~Modified:	1/13/2012																	*
******************************************************************************************~*/

package
{
	import defines.Assets;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	
	import ui.MuteSoundsBtn;
	
	public class LevelSelect extends World
	{
		private const NUM_COLS:int = 5;
		private const NUM_ROWS:int = 3;
				
		private const BTN_PADDING_X:Number = 128;
		private const BTN_PADDING_Y:Number = 72;				//72 for 3 rows			128 for 2 rows
		private const BTN_LFTCOL:Number = 128;
		private const BTN_TOPROW:Number = 256 + 64;			//256+64 for 3 rows		256+128 for 2 rows
		private const HIT_DISTANCE:Number = 32;
		
		private const COLOR_SELECTED:uint = 0xFF0077AA;
		private const COLOR_UNLOCKED:uint = 0xFFFFFFFF;
		private const COLOR_LOCKEDUP:uint = 0xFF404050;
		
		private var mGameTitle:Image = new Image(Assets.GAME_TITLE, new Rectangle(0, 0, 800, 300));
		private var mButtonBorder:Image = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(0, 0, 64, 64));
		private var mButtonText:Text = new Text("1");
		private var mUnlockText:Text = new Text("Press 'U' to unlock levels.");
		private var mOverCol:int = -1;
		private var mOverRow:int = -1;

		public function LevelSelect():void
		{
			//Hackery, since we don't have menu music- begin it now.
			if (false == GameWorld.mGameMusic.playing)
				GameWorld.mGameMusic.loop();
			
			mButtonBorder.centerOrigin();
			mButtonText.scale = 1.5;
			
			mUnlockText.x = FP.screen.width / 2.0;
			mUnlockText.y = 580;
			mUnlockText.centerOrigin();
			addGraphic(mUnlockText);
			
			FP.screen.color = 0xFF000000;
			
			addGraphic(mGameTitle);
			add(new MuteSoundsBtn());
		}

		private function xGridAt(col:int):Number { return BTN_LFTCOL + (BTN_PADDING_X * col); }
		private function yGridAt(row:int):Number { return BTN_TOPROW + (BTN_PADDING_Y * row); }
		
		override public function render():void
		{
			super.render();
			
			for (var r:int = 0; r < NUM_ROWS; ++r)
			{
				for (var c:int = 0; c < NUM_COLS; ++c)
				{
					var atX:Number = xGridAt(c);
					var atY:Number = yGridAt(r);
					var lvl:int = (c + 1) + (r * NUM_COLS);
					
					if (lvl <= GameWorld.UnlockedLevels || Input.check(Key.U))
					{
						if (mOverCol == c && mOverRow == r)
						{
							mButtonBorder.color = COLOR_SELECTED;
							mButtonText.color = COLOR_SELECTED;
						}
						else
						{
							mButtonBorder.color = COLOR_UNLOCKED;
							mButtonText.color = COLOR_UNLOCKED;
						}
					}
					else
					{	//Level locked.
						mButtonBorder.color = COLOR_LOCKEDUP;
						mButtonText.color = COLOR_LOCKEDUP;
					}
					
					mButtonBorder.render(FP.buffer, new Point(atX, atY), camera);
					mButtonText.text = lvl.toString();
					mButtonText.render(FP.buffer, new Point(atX - (mButtonText.width * 0.5), atY - (mButtonText.height * 0.5)), camera);
				}
			}
			
			//mButtonText.text = ;
			//mButtonText.render(FP.buffer, new Point((FP.screen.width / 2.0) + (mButtonText.width / 2.0), 550), camera);
		}
		
		override public function update():void
		{
			super.update();
			
			if (true == Assets.IS_DEV_MODE && Input.pressed(Key.E))
				FP.world = new GameEditor();
				
			mOverCol = mOverRow = -1;
			for (var r:int = 0; r < NUM_ROWS; ++r)
			{
				for (var c:int = 0; c < NUM_COLS; ++c)
				{
					var atX:Number = xGridAt(c);
					var atY:Number = yGridAt(r);
					
					if (FP.distance(atX, atY, mouseX, mouseY) < HIT_DISTANCE)
					{
						mOverCol = c;
						mOverRow = r;
						
						if (true == Input.mousePressed)
						{
							var lvl:int = (c + 1) + (r * NUM_COLS);
							if (lvl <= GameWorld.UnlockedLevels || Input.check(Key.U))
							{
								Assets.PlaySoundIn(Assets.inSfxButtonClick);
								FP.world = new GameLevels();
								GameWorld.GetInstance().LoadLevel(lvl);
							}
							else { /* TODO: Play negative click sound. */ }
						}
					}
				}
			}
		}
	}
}