package
{
	import defines.Assets;
	import flash.geom.Rectangle;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.Sfx;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	
	public class TinyKid extends Entity
	{
		private const MAX_DEATH_TIMER:Number = 2.0;
		private const GRAB_LIFT_AMOUNT:Number = 8.0;
		
		//private static const mSfxOnDeath:Array = new Array(new Sfx(Assets.SFX_TINYKID_ONDEATH), new Sfx(Assets.SFX_TINYKID_ONDEATH));
		//private static const mSfxOnSafety:Array = new Array(new Sfx(Assets.SFX_TINYKID_ONSAFETY), new Sfx(Assets.SFX_TINYKID_ONSAFETY));
		private static const mSfxOnDeath:Sfx = new Sfx(Assets.SFX_TINYKID_ONDEATH);
		private static const mSfxOnSafety:Sfx = new Sfx(Assets.SFX_TINYKID_ONSAFETY);
		
		private var mSpriteMap:Spritemap = new Spritemap(Assets.TINYKID_SHEET, 24, 24, OnAnimComplete);
		private var mDropShadow:Image = new Image(Assets.TINYKID_SHEET, new Rectangle(96, 0, 24, 24));
		private var mAliveTimer:Number = -1;
		private var mDeathTimer:Number = -1;
		
		public function TinyKid(atX:Number = 0, atY:Number = 500)
		{
			x = atX;	y = atY;
			mSpriteMap.centerOrigin();
			mSpriteMap.add("idle", [0, 1, 2], 2, true);
			mSpriteMap.add("wiggle", [16, 17, 18, 19], 14, true);
			mSpriteMap.add("fall", [8, 9, 10, 11], 15, true);
			mSpriteMap.add("safety", [24, 25, 26, 27, 28, 29, 30, 31], 15, false);
			mSpriteMap.play("idle");
			
			mDropShadow.centerOrigin();		//mDropShadow.originY -= 9;
			mDropShadow.visible = false;
			addGraphic(mDropShadow);
			addGraphic(mSpriteMap);
		}
		
		public override function update():void
		{
			super.update();

			if (mDeathTimer > 0.0)
			{
				mDeathTimer -= FP.elapsed;
				if (mDeathTimer < 0.0)
				{
					world.remove(this);
					GameWorld.WaitToFinish--;
				}
				
				if (mDeathTimer < 1.0)
				{
					mSpriteMap.scale = Math.max(0.2, mDeathTimer);
				}
			}
			
			if (mAliveTimer > 0.0)
			{
				mAliveTimer -= FP.elapsed;
				if (mAliveTimer < 0.0)
				{
					world.remove(this);
					GameWorld.WaitToFinish--;
				}
				
				x -= 35 * FP.elapsed;
			}
		}
		
		public function OnGrab():void
		{
			mSpriteMap.play("wiggle");
			mDropShadow.visible = true;
			mSpriteMap.originY += GRAB_LIFT_AMOUNT;
		}
		
		public function OnUngrab(moved:Boolean, extra:Boolean):void
		{
			mDropShadow.visible = false;
			mSpriteMap.originY -= GRAB_LIFT_AMOUNT;
			if (mSpriteMap.currentAnim != "fall" && mSpriteMap.currentAnim != "safety")
				mSpriteMap.play("idle");
				
			if (true == moved)
			{
				if (true == extra)
				{	Assets.PlaySoundIn(Assets.inSfxOnUngrabKid); 	}	//Cost of move = 1
				else
				{	Assets.PlaySoundIn(Assets.inSfxOnUngrabKid);	}	//Cost of move > 1
			}
			else
			{
				if (true == extra)
				{	Assets.PlaySoundIn(Assets.inSfxOnUngrabKid);	}	//Kid was not ordered to go somewhere new.
				else
				{	Assets.PlaySoundIn(Assets.inSfxOnUngrabKid);	}	//Kid could not go where ordered.
			}
		}
		
		public function OnDeath():void
		{
			mDeathTimer = MAX_DEATH_TIMER;
			mSpriteMap.play("fall");
			mSpriteMap.scale = 1.25;
			
			if (false == mSfxOnDeath.playing)
				mSfxOnDeath.play();		//TODO: maybe delay, or add some silence at the start.
		}
		
		public function OnReachSafety():void
		{
			mAliveTimer = 1.0;
			mSpriteMap.play("safety");
			x += 12;
			if (false == mSfxOnSafety.playing)
				mSfxOnSafety.play();
		}
		
		public function OnAnimComplete():void
		{
			//if ("safety" == mSpriteMap.currentAnim)
			//{
//
			//}
		}
	}
}