/*~******************************************************************************************
*	~Project:	"BusTown"																	*
*	~Author:	Tim "BlackBird" Beaudet														*
*	~Modified:	1/13/2012																	*
******************************************************************************************~*/

package map
{
	import defines.Assets;
	import defines.TileTypes;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.net.FileReference;
	import flash.system.System;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Text;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.World;
	import net.flashpunk.FP;
	

	public class HexagonMap extends Graphic
	{
		public static const NUM_TILE_TYPES:int = TileTypes.MaxTypes;
		
		private var mSnipEdge:Boolean = true;				//True to make the left/right edges symetrical.
		private var mNumCols:int;
		private var mNumRows:int;
		
		//Defines the size of teach of the tiles.
		private var mHexagonS:Number;
		private var mHexagonH:Number;
		private var mHexagonR:Number;
		private const mHexagonPoints:Array = new Array();	//Used for the collision
		
		private var mMapData:Array = new Array();			//Array of MapInfo's
		private var mTileSet:*;
		private var mTileImages:Array = new Array();
		
		public function HexagonMap(tileSet:*, sideLength:Number = 32.0, numCols:int = 13, numRows:int = 9):void
		{
			active = true;
			
			mTileSet = tileSet;
			mHexagonS = sideLength;
			mHexagonH = Math.sin((30.0 / 180.0) * Math.PI) * mHexagonS;
			mHexagonR = Math.cos((30.0 / 180.0) * Math.PI) * mHexagonS;
			
			for (var im:int = 0; im < 64; ++im)
				mTileImages[im] = null;
				
			mTileImages[0] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(0, 0, 64, 64));
			mTileImages[1] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(64, 0, 64, 64));
			mTileImages[2] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(128, 0, 64, 64));
			mTileImages[3] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(192, 0, 64, 64));
			mTileImages[4] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(256, 0, 64, 64));

			//Destruction Stage 1
			mTileImages[9] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(64, 64, 64, 64));
			mTileImages[10] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(128, 64, 64, 64));
			mTileImages[11] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(192, 64, 64, 64));
			mTileImages[12] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(256, 0, 64, 64));

			//Destruction Stage 2
			mTileImages[17] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(64, 128, 64, 64));
			mTileImages[18] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(128, 128, 64, 64));
			mTileImages[19] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(192, 128, 64, 64));
			mTileImages[20] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(256, 0, 64, 64));
			
			//Destruction Stage 3
			mTileImages[25] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(64, 192, 64, 64));
			mTileImages[26] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(128, 192, 64, 64));
			mTileImages[27] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(192, 192, 64, 64));
			mTileImages[28] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(256, 0, 64, 64));
			
			//Overlay Tiles
			mTileImages[8] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(0, 64, 64, 64));
			mTileImages[16] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(0, 128, 64, 64));
			mTileImages[24] = new Image(Assets.HEXAGON_TILESET_SD, new Rectangle(0, 192, 64, 64));
			
			for (var i:int = 0; i < mTileImages.length; ++i)
			{
				if (null == mTileImages[i]) continue;
				mTileImages[i].originX = mTileImages[i].width / 2.0;
				mTileImages[i].originY = mTileImages[i].height / 2.0;
			}
			
			//Set the entire map to the default tile type: 0
			mNumCols = numCols;		mNumRows = numRows;
			for (var row:int = 0; row < mNumRows; ++row)
			{
				for (var col:int = 0; col < mNumCols; ++col)
				{
					//mMapData[col + (row * mNumCols)] = 0;
					mMapData[mapIndexOf(col, row)] = new MapInfo();
				}
			}
		}
		
		public function SetAllTilesTo(newType:int = 0, newSubType:int = -1):void
		{
			for (var idx:int = 0; idx < mNumCols * mNumRows; ++idx)
			{
				mMapData[idx].type = newType;
				mMapData[idx].subType = newSubType;
			}
		}
		
		
		//Returns the _center_ coordinates of the hexagon tile.
		//Maybe rename to: xCenterOfTile(col, row)
		public function xGridPositionOf(col:int, row:int):Number { return (((col * 2) * mHexagonR) + ((row % 2) * mHexagonR)) + mHexagonR + 40; }
		public function yGridPositionOf(row:int):Number { return (row * (mHexagonH + mHexagonS)) + mHexagonH * 2.1; }
		public function mapIndexOf(col:int, row:int):int { return col + (row * mNumCols); }
		private function SnipEdge(col:int, row:int):Boolean	{	return (true == mSnipEdge && col == mNumCols - 1 && row % 2) ? true : false;	}

		public function GetNumCols():int { return mNumCols; }
		public function GetNumRows():int { return mNumRows; }
		public function GetTileInfo(idx:int):MapInfo { return mMapData[idx]; }
		public function SetTileType(idx:int, newType:int):void { mMapData[idx].type = newType; }
		public function SetTileSubType(idx:int, newType:int):void { mMapData[idx].subType = newType; }
		public function SetTileLife(idx:int, newLife:int):void { mMapData[idx].life = (newLife < -1) ? -1 : newLife; }

		public function GetTileImageOf(type:int, life:int):Image
		{
			var damage:int = (life >= 4 || life < 0) ? 0 : 4 - life;
			return mTileImages[type + (damage * 8)];
		}
		
		public function GetTileImage(tile:HexagonTile):Image
		{
			var type:int = mMapData[tile.idx].type;
			var life:int = mMapData[tile.idx].life;
			return GetTileImageOf(type, life);
		}

		public function GetTileImageAt(col:int, row:int):Image
		{
			var type:int = mMapData[mapIndexOf(col, row)].type;
			var life:int = mMapData[mapIndexOf(col, row)].life;
			return GetTileImageOf(type, life);
		}
		
		public function CollidesWithTile(atPt:Point, col:int, row:int):Boolean
		{
			if (0 == mHexagonPoints.length)
			{	//These points are based from the _CENTER_ of the hexagon!
				mHexagonPoints[0] = 0.0;			mHexagonPoints[1] = -((mHexagonS * 0.5) + mHexagonH);
				mHexagonPoints[2] = mHexagonR;		mHexagonPoints[3] = -mHexagonS * 0.5;
				mHexagonPoints[4] = mHexagonR;		mHexagonPoints[5] = mHexagonS * 0.5;
				mHexagonPoints[6] = 0.0;			mHexagonPoints[7] = ((mHexagonS * 0.5) + mHexagonH);
				mHexagonPoints[8] = -mHexagonR;		mHexagonPoints[9] = mHexagonS * 0.5;
				mHexagonPoints[10] = -mHexagonR;	mHexagonPoints[11] = -mHexagonS * 0.5;
			}
			
			var xPt:Number = atPt.x - xGridPositionOf(col, row);
			var yPt:Number = atPt.y - yGridPositionOf(row);
			
			//Check if xPt,yPt falls within the mHexagonPoints.
			for (var pt:int = 0; pt < 6; ++pt)
			{
				var npt:int = (pt + 1) % 6;
				
				var xNorm:Number = mHexagonPoints[(npt * 2) + 0] - mHexagonPoints[(pt * 2) + 0];
				var yNorm:Number = mHexagonPoints[(npt * 2) + 1] - mHexagonPoints[(pt * 2) + 1];
				var diff:Number = Math.sqrt((xNorm * xNorm) + (yNorm * yNorm));
				xNorm /= diff;		yNorm /= diff;
				
				var xPtDiff:Number = mHexagonPoints[(pt * 2) + 0] - xPt;
				var yPtDiff:Number = mHexagonPoints[(pt * 2) + 1] - yPt;
				
				//Swap xNorm and -yNorm to make a normal pointing OUT of the hexagon.
				if ((xPtDiff * -yNorm) + (yPtDiff * xNorm) >= 0.0)
					return false;
			}
			
			return true;
		}
		
		public function GetTileAt(col:int, row:int):HexagonTile
		{
			var tile:HexagonTile = new HexagonTile();
			tile.col = col;		tile.row = row;			tile.idx = mapIndexOf(col, row);
			tile.pos.x = xGridPositionOf(col, row);		tile.pos.y = yGridPositionOf(row);
			tile.type = mMapData[tile.idx].type;
			return tile;
		}
		
		public function CollisionWithMap(pt:Point):HexagonTile
		{	//OPTIMIZE: Narrow down the search with smarter maths based on position of pt.
			for (var row:int = 0; row < mNumRows; ++row)
			{
				for (var col:int = 0; col < mNumCols; ++col)
				{
					if (true == SnipEdge(col, row))
						continue;
					
					if (true == CollidesWithTile(pt, col, row))
					{
						return GetTileAt(col, row);
					}
				}
			}
			
			return null;
		}
		
		public function MakeFromArray(data:Array):void
		{
			if (data.length == mNumCols * mNumRows)
			{
				for (var row:int = 0; row < mNumRows; ++row)
				{
					for (var col:int = 0; col < mNumCols; ++col)
					{
						mMapData[mapIndexOf(col, row)] = data[mapIndexOf(col, row)];
					}
				}
			}
		}
		
		override public function update():void
		{
			super.update();
			var deltaTime:Number = FP.elapsed;
		}
		
		override public function render(target:BitmapData, point:Point, camera:Point):void
		{
			super.render(target, point, camera);
			
			var x:Number = xGridPositionOf(2, 2);
			var y:Number = yGridPositionOf(2);
			
			for (var row:int = 0; row < mNumRows; ++row)
			{
				for (var col:int = 0; col < mNumCols; ++col)
				{
					if (SnipEdge(col, row))
						continue;
						
					var type:int = mMapData[mapIndexOf(col, row)].type;
					var life:int = mMapData[mapIndexOf(col, row)].life;
					if (life == 0 || (false == Assets.IS_DEV_MODE && type == TileTypes.Empty))
						continue;
						
					var pt:Point = new Point();
					pt.x = point.x + xGridPositionOf(col, row);
					pt.y = point.y + yGridPositionOf(row);
					
					GetTileImageOf(type, life).render(target, pt, camera);
				}
			}
		}
		
		
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		public function editorLoadDefault():void
		{
			for (var idx:int = 0; idx < mNumCols * mNumRows; ++idx)
				mMapData[idx] = new MapInfo();
		}
		
		public function editorSaveString():String
		{
			var mapDataAsString:String = new String();
			mapDataAsString += "mWorldMap.MakeFromArray(new Array(";// + mMapData.join(", ") + "));\n"
			
			for (var idx:int = 0; idx < mNumCols * mNumRows; ++idx)
			{
				if (idx != 0)
					mapDataAsString += ", "
				var info:MapInfo = mMapData[idx];
				mapDataAsString += "new MapInfo(" + info.type + ", " + info.subType + ", " + info.life + ", " + info.kids + ")";
			}
			mapDataAsString += "));\n";
			return mapDataAsString;
		}

		public function renderEditor(target:BitmapData, point:Point, camera:Point):void
		{
			super.render(target, point, camera);
			
			var x:Number = xGridPositionOf(2, 2);
			var y:Number = yGridPositionOf(2);
			
			for (var row:int = 0; row < mNumRows; ++row)
			{
				for (var col:int = 0; col < mNumCols; ++col)
				{
					if (SnipEdge(col, row))
						continue;
						
					var type:int = mMapData[mapIndexOf(col, row)].type;
					var pt:Point = new Point();
					pt.x = point.x + xGridPositionOf(col, row);
					pt.y = point.y + yGridPositionOf(row);
										
					var txt:Text = new Text(mMapData[mapIndexOf(col, row)].life, pt.x, pt.y);
					txt.render(target, point, camera);
				}
			}
		}
	}
}



